pub trait ToSymbol {
// Required method
fn symbol(&self) -> Result<Symbol, ClingoError>;
}Expand description
helper types and traits to simplify conversion from structs to clingo symbols
pub trait ToSymbol {
// Required method
fn symbol(&self) -> Result<Symbol, ClingoError>;
}helper types and traits to simplify conversion from structs to clingo symbols